﻿
'NXJournaling.com
'August 9, 2019

'goal seek using the bisection method
'this journal will change the value of a given expression (within the specified bounds)
'until a given value is met
'in the example file, the chord length is varied until the area bounded by the 2 arcs meets the specified value
'example file: area_test.prt

Option Strict Off
Imports System
Imports NXOpen
Imports NXOpen.UF

Module Module2

    Dim theSession As Session = Session.GetSession()
    Dim theUfSession As UFSession = UFSession.GetUFSession()

    Dim theUI As UI = UI.GetUI()
    Dim lw As ListingWindow = theSession.ListingWindow

    Sub Main()

        Dim markId1 As Session.UndoMarkId
        markId1 = theSession.SetUndoMark(Session.MarkVisibility.Visible, "NXJ goal seek")

        lw.Open()

        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
        'Change the values of the following constants/variables to suit your needs
        Const expressionName As String = "chord_length"
        Const measureExpressionName As String = "p9"
        Const targetValue As Double = 12.25
        Dim lowerBound As Double = 5
        Dim upperBound As Double = 13
        Const tolerance As Double = 0.0001
        '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        Dim theExpression As Expression
        Try
            theExpression = theSession.Parts.Work.Expressions.FindObject(expressionName)
        Catch ex As NXException
            If ex.ErrorCode = 3520016 Then
                'no expression found with given name
                lw.WriteLine("expression '" & expressionName & "' not found, journal exiting")
            Else
                lw.WriteLine(ex.ErrorCode & ": " & ex.Message)
            End If
            Return
        End Try

        Dim measureExpression As Expression
        Try
            measureExpression = theSession.Parts.Work.Expressions.FindObject(measureExpressionName)
        Catch ex As NXException
            If ex.ErrorCode = 3520016 Then
                'no expression found with given name
                lw.WriteLine("expression '" & measureExpressionName & "' not found, journal exiting")
            Else
                lw.WriteLine(ex.ErrorCode & ": " & ex.Message)
            End If
            Return
        End Try

        'check that lowerBound yields value lower than target
        If UpdatedValue(lowerBound, theExpression, measureExpression) > targetValue Then
            lw.WriteLine("lower bound is set too high")
            Return
        End If

        'check that upperBound yields value higher than target
        If UpdatedValue(upperBound, theExpression, measureExpression) < targetValue Then
            'lw.WriteLine("upperbound: " & upperBound.ToString)
            'lw.WriteLine("area: " & UpdatedValue(upperBound, theExpression, measureExpression).ToString)
            lw.WriteLine("upper bound is set too low")
            Return
        End If

        Dim currentInput As Double = lowerBound + (upperBound - lowerBound) / 2
        Dim calculatedValue As Double = UpdatedValue(currentInput, theExpression, measureExpression)

        DoUpdate()
        'Return
        Do Until Math.Abs(targetValue - calculatedValue) <= tolerance
            'lw.WriteLine("currentInput: " & currentInput.ToString)
            'lw.WriteLine("calculatedValue: " & calculatedValue.ToString)
            If calculatedValue < targetValue Then
                lowerBound = currentInput
            Else
                upperBound = currentInput
            End If
            currentInput = lowerBound + (upperBound - lowerBound) / 2
            calculatedValue = UpdatedValue(currentInput, theExpression, measureExpression)
            DoUpdate()
        Loop

        lw.WriteLine(theExpression.Name & " = " & currentInput.ToString)
        lw.WriteLine("area = " & measureExpression.Value.ToString)

        lw.Close()

    End Sub

    Function UpdatedValue(ByVal input As Double, inputExpression As Expression, outputExpression As Expression) As Double

        theSession.UpdateManager.ClearErrorList()
        Dim markId2 As Session.UndoMarkId
        markId2 = theSession.SetUndoMark(Session.MarkVisibility.Invisible, "Expression edit")

        inputExpression.RightHandSide = input.ToString

        'update model
        Try
            DoUpdate()

        Catch ex As NXException
            lw.WriteLine("** Update Error with value: " & input.ToString)
            lw.WriteLine("** " & ex.ErrorCode & ": " & ex.Message)
            lw.WriteLine("")

        End Try

        Return outputExpression.Value

    End Function

    Sub DoUpdate()

        theSession.UpdateManager.ClearErrorList()
        Dim markId2 As Session.UndoMarkId
        markId2 = theSession.SetUndoMark(Session.MarkVisibility.Invisible, "Model update")

        Dim nErrs1 As Integer

        Try
            nErrs1 = theSession.UpdateManager.DoUpdate(markId2)

        Catch ex As NXException
            Throw ex

        End Try

    End Sub

    Public Function GetUnloadOption(ByVal dummy As String) As Integer

        'Unloads the image immediately after execution within NX
        GetUnloadOption = NXOpen.Session.LibraryUnloadOption.Immediately

    End Function

End Module
